@@ -9,8 +9,8 @@ pylibftdi: http://bitbucket.org/codedstructure/pylibftdi
 """
 
 import os
+import itertools
 import functools
-import warnings
 
 # be disciplined so pyflakes can check us...
 from ctypes import (CDLL, byref, c_int, c_char_p, c_void_p, cast,
@@ -37,9 +37,16 @@ class UsbDevList(Structure):
 FLUSH_BOTH = 1
 FLUSH_INPUT = 2
 FLUSH_OUTPUT = 3
+# Device Modes
+BITMODE_RESET = 0x00
+BITMODE_BITBANG = 0x01
 
+# Default USB IDs
 USB_VID = 0x0403
 USB_PID = 0x6001
+# However, a list of IDs is actually matched against.
+USB_VID_LIST = [USB_VID]
+USB_PID_LIST = [USB_PID, 0x6014]
 
 
 class Driver(object):
@@ -74,39 +81,6 @@ class Driver(object):
             self.fdll = fdll
         self._need_init = False
 
-    ## Legacy support - prior to version 0.7, Device and Driver
-    ## were unified (and named 'Driver'). This provides basic
-    ## backwards compatiblity support.
-    LEGACY_ATTRIBUTES = ['open', 'close', 'ftdi_fn', 'baudrate',
-                         'read', 'write', 'get_error_string',
-                         '__enter__', '__exit__']
-
-    @property
-    def legacy_device(self):
-        warnings.warn("using Device() methods on Driver(); see CHANGES.txt",
-                DeprecationWarning)
-        if not hasattr(self, '_legacy_device'):
-            self._legacy_device = Device(lazy_open=True)
-        return self._legacy_device
-
-    def __getattr__(self, key):
-        if key in Driver.LEGACY_ATTRIBUTES:
-            return getattr(self.legacy_device, key)
-        else:
-            return object.__getattr__(self, key)
-
-    def __setattr__(self, key, value):
-        if key in Driver.LEGACY_ATTRIBUTES:
-            return setattr(self.legacy_device, key, value)
-        else:
-            self.__dict__[key] = value
-
-    def __delattr__(self, key):
-        if key in Driver.LEGACY_ATTRIBUTES:
-            delattr(self.legacy_device, key)
-        else:
-            del self.__dict__[key]
-
     def list_devices(self):
         """
         return a list of triples (manufacturer, description, serial#)
@@ -135,26 +109,28 @@ class Driver(object):
             raise FtdiError(msg)
 
         try:
-            res = self.fdll.ftdi_usb_find_all(byref(ctx),
-                                              byref(dev_list_ptr),
-                                              USB_VID, USB_PID)
-            if res < 0:
-                raise FtdiError(self.fdll.ftdi_get_error_string(byref(ctx)))
-            elif res > 0:
-                # take a copy of the dev_list for subsequent list_free
-                dev_list_base = pointer(dev_list_ptr.contents)
-                # traverse the linked list...
-                try:
-                    while dev_list_ptr:
-                        self.fdll.ftdi_usb_get_strings(byref(ctx),
-                                dev_list_ptr.contents.usb_dev,
-                                manuf, 127, desc, 127, serial, 127)
-                        devices.append((manuf.value, desc.value, serial.value))
-                        # step to next in linked-list if not
-                        dev_list_ptr = cast(dev_list_ptr.contents.next,
-                                            devlistptrtype)
-                finally:
-                    self.fdll.ftdi_list_free(dev_list_base)
+            for usb_vid, usb_pid in itertools.product(USB_VID_LIST, USB_PID_LIST):
+                res = self.fdll.ftdi_usb_find_all(byref(ctx),
+                                                byref(dev_list_ptr),
+                                                usb_vid,
+                                                usb_pid)
+                if res < 0:
+                    raise FtdiError(self.fdll.ftdi_get_error_string(byref(ctx)))
+                elif res > 0:
+                    # take a copy of the dev_list for subsequent list_free
+                    dev_list_base = pointer(dev_list_ptr.contents)
+                    # traverse the linked list...
+                    try:
+                        while dev_list_ptr:
+                            self.fdll.ftdi_usb_get_strings(byref(ctx),
+                                    dev_list_ptr.contents.usb_dev,
+                                    manuf, 127, desc, 127, serial, 127)
+                            devices.append((manuf.value, desc.value, serial.value))
+                            # step to next in linked-list if not
+                            dev_list_ptr = cast(dev_list_ptr.contents.next,
+                                                devlistptrtype)
+                    finally:
+                        self.fdll.ftdi_list_free(dev_list_base)
         finally:
             self.fdll.ftdi_deinit(byref(ctx))
         return devices
@@ -183,7 +159,8 @@ class Device(object):
       user must perform an explicit open() call prior to other operations.
     """
     def __init__(self, device_id=None, mode="b",
-                 encoding="latin1", lazy_open=False):
+                 encoding="latin1", lazy_open=False,
+                 buffer_size=0):
         self._opened = False
         self.driver = Driver()
         self.fdll = self.driver.fdll
@@ -195,7 +172,7 @@ class Device(object):
         # strings; for binary the raw bytes will be returned.
         # This will only affect Python3.
         self.mode = mode
-        # when giving a str to Driver.write(), it is encoded.
+        # when giving a str to Device.write(), it is encoded.
         # default is latin1, because it provides
         # a one-to-one correspondence for code points 0-FF
         self.encoding = encoding
@@ -205,12 +182,15 @@ class Device(object):
         self._baudrate = 9600
         # defining softspace allows us to 'print' to this device
         self.softspace = 0
+        # buffer_size (if not 0) chunks the reads and writes
+        # to allow interruption
+        self.buffer_size = buffer_size
         # lazy_open tells us not to open immediately.
         if not lazy_open:
             self.open()
 
     def __del__(self):
-        "tell driver to free the ftdi_context resource"
+        "free the ftdi_context resource"
         if self._opened:
             self.close()
 
@@ -233,18 +213,21 @@ class Device(object):
         # they were, but we can't use self.close as that assumes things
         # have already been setup.
         # FTDI vendor/product ids required here.
-        open_args = [byref(self.ctx), USB_VID, USB_PID]
-        if self.device_id is None:
-            res = self.fdll.ftdi_usb_open(*tuple(open_args))
-        else:
-            # attempt to match device_id to serial number
-            open_args.extend([0, c_char_p(self.device_id.encode('latin1'))])
-            res = self.fdll.ftdi_usb_open_desc(*tuple(open_args))
-            if res != 0:
-                # swap last two parameters and try again
-                #  - attempt to match device_id to description
-                open_args[-2], open_args[-1] = open_args[-1], open_args[-2]
+        for usb_vid, usb_pid in itertools.product(USB_VID_LIST, USB_PID_LIST):
+            open_args = [byref(self.ctx), usb_vid, usb_pid]
+            if self.device_id is None:
+                res = self.fdll.ftdi_usb_open(*tuple(open_args))
+            else:
+                # attempt to match device_id to serial number
+                open_args.extend([0, c_char_p(self.device_id.encode('latin1'))])
                 res = self.fdll.ftdi_usb_open_desc(*tuple(open_args))
+                if res != 0:
+                    # swap last two parameters and try again
+                    #  - attempt to match device_id to description
+                    open_args[-2], open_args[-1] = open_args[-1], open_args[-2]
+                    res = self.fdll.ftdi_usb_open_desc(*tuple(open_args))
+            if res == 0:
+                break
 
         if res != 0:
             msg = "%s (%d)" % (self.get_error_string(), res)
@@ -253,6 +236,10 @@ class Device(object):
             del self.ctx
             raise FtdiError(msg)
 
+        # explicitly reset the device to serial mode in case
+        # it had previously been used in bitbang mode
+        # (some later driver versions might do this automatically)
+        self.fdll.ftdi_set_bitmode(byref(self.ctx), 0, BITMODE_RESET)
         self._opened = True
 
     def close(self):
@@ -303,7 +290,18 @@ class Device(object):
             raise FtdiError("read() on closed Device")
 
         # read the data
-        byte_data = self._read(length)
+        if self.buffer_size != 0:
+            remaining = length
+            byte_data_list = []
+            while remaining > 0:
+                rx_bytes = self._read(min(remaining, self.buffer_size))
+                if not rx_bytes:
+                    break
+                byte_data_list.append(rx_bytes)
+                remaining -= len(rx_bytes)
+            byte_data = b''.join(byte_data_list)
+        else:
+            byte_data = self._read(length)
         if self.mode == 'b':
             return byte_data
         else:
@@ -339,7 +337,25 @@ class Device(object):
             byte_data = data.encode(self.encoding)
 
         # actually write it
-        written = self._write(byte_data)
+        if self.buffer_size != 0:
+            remaining = len(byte_data)
+            written = 0
+            while remaining > 0:
+                start = written
+                length = min(remaining, self.buffer_size)
+                result = self._write(byte_data[start: start + length])
+                if result == -1:
+                    written == result
+                    break
+                elif result == 0:
+                    # don't continue to try writing forever if nothing
+                    # is actually being written
+                    break
+                else:
+                    written += result
+                    remaining -= result
+        else:
+            written = self._write(byte_data)
         if written == -1:
             raise FtdiError(self.get_error_string())
         return written
@@ -411,11 +427,11 @@ class Device(object):
     def __enter__(self):
         """
         support for context manager.
-        Note the driver is opened and closed automatically
-        when used in a with statement, and the driver object
+        Note the device is opened and closed automatically
+        when used in a with statement, and the device object
         itself is returned:
-        >>> with Driver(mode='t') as drv:
-        >>>     drv.write('Hello World!')
+        >>> with Device(mode='t') as dev:
+        >>>     dev.write('Hello World!')
         >>>
         """
         self.open()
